import argparse

from obwsc.commands.event_based_command import EventBasedCommand

from obwsc.log import Log


class SetCurrentProfile(EventBasedCommand):
    def __init__(self, config, profile: str):
        super().__init__(config)
        self.events.callback.register(self.on_current_profile_changed)
        self.target_profile = profile

    @staticmethod
    def get_parser_name():
        return 'set-current-profile'

    @staticmethod
    def get_help():
        return 'Set the current OBS profile. This can only be done when no recording is active.'

    @staticmethod
    def add_arguments(parser: argparse.ArgumentParser):
        parser.add_argument('profile', type=str, help='The name of the OBS profile to switch to')

    @staticmethod
    def parse_arguments(args) -> dict:
        return {'profile': args.profile}

    def on_current_profile_changed(self, event):
        if event.profile_name == self.target_profile:
            self.done.set()

    def execute(self):
        profiles = self.ws.get_profile_list()
        if self.target_profile not in profiles.profiles:
            raise RuntimeError(f'Profile "{self.target_profile}" does not exist')

        if profiles.current_profile_name == self.target_profile:
            Log.debug('Profile already active')
            return

        status = self.ws.get_record_status()
        if status.output_active:
            raise RuntimeError('Profile cannot be changed while recording is active')

        self.ws.set_current_profile(self.target_profile)
        self.done.wait()



import obsws_python as obs

import argparse

from threading import Event

from obwsc.log import Log


class EventBasedCommand:
    """
    Base class for commands that have the following workflow:
    1. Send a request to OBS
    2. Wait for a specific event from OBS
    3. Complete
    """

    def __init__(self, config):
        self.ws = obs.ReqClient(**config)
        self.ws.logger.disabled = True  # We don't need any logs from the WS library
        self.events = obs.EventClient(**config)
        self.events.callback.register(self.on_exit_started)
        self.done = Event()

    def on_exit_started(self, _):
        self.events.unsubscribe()
        Log.warning('OBS is shutting down')
        self.done.set()

    def execute(self):
        raise NotImplementedError()

    @staticmethod
    def get_parser_name():
        return None

    @staticmethod
    def get_help():
        return None

    @staticmethod
    def add_arguments(parser: argparse.ArgumentParser):
        pass

    @staticmethod
    def parse_arguments(_) -> dict:
        return {}

    def __enter__(self):
        self.ws.__enter__()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.ws.__exit__(exc_type, exc_val, exc_tb)
        self.events.unsubscribe()



#!/usr/bin/env python3
import inspect
import pkgutil
import sys
import argparse

from obsws_python.error import OBSSDKError

from obwsc.options import Options
from obwsc.commands.event_based_command import EventBasedCommand

import obwsc.commands

from typing import Optional, List


def get_classes(package):
    imported = {}
    prefix = package.__name__ + "."
    for importer, modname, is_package in pkgutil.iter_modules(package.__path__, prefix):
        if is_package:
            continue

        imported[modname] = __import__(modname, fromlist='main')

    return imported


def collect_commands_from_modules(modules):
    res = set()
    for key in modules:
        module = modules[key]
        for name, obj in inspect.getmembers(module):
            if not inspect.isclass(obj):
                continue

            if name == 'EventBasedCommand':
                continue

            if issubclass(obj, EventBasedCommand):
                res.add(obj)

    return [x for x in res]


def collect_commands():
    # noinspection PyTypeChecker
    return collect_commands_from_modules(get_classes(package=obwsc.commands))


def main(cmd_args: Optional[List[str]] = None):
    commands = collect_commands()
    parsers = {}
    arg_parser = None  # type: Optional[argparse.ArgumentParser]

    def add_command_parsers(parser):
        nonlocal arg_parser, parsers
        arg_parser = parser

        subparsers = parser.add_subparsers(title='command', dest='command', help='Command to execute')
        for cmd in commands:
            name = cmd.get_parser_name()
            hlp = cmd.get_help()
            if name is None or hlp is None:
                continue

            if name in parsers:
                raise RuntimeError(f'Duplicate command name: {name}, already used by "{parsers[name]}"')

            parsers[name] = cmd
            command_parser = subparsers.add_parser(name, help=hlp)
            cmd.add_arguments(command_parser)

    config, args = Options.parse('obws-command', extra_args_fn=add_command_parsers, cmd_args=cmd_args)

    if args.command is None:
        arg_parser.print_usage(sys.stderr)
        raise RuntimeError('Missing required argument: command')

    obs_config = config['obs']

    command_type = parsers[args.command]
    extra_args = command_type.parse_arguments(args)
    with command_type(obs_config, **extra_args) as command:
        result = command.execute()
        if result is not None:
            print(result)


def run(cmd_args: Optional[List[str]] = None):
    try:
        main(cmd_args)
        sys.exit(0)
    except OBSSDKError as e:
        print(e)
    except ConnectionRefusedError as e:
        print(f'Connection refused, please make sure OBS is running', file=sys.stderr)
    except RuntimeError as e:
        print(e, file=sys.stderr)

    sys.exit(-1)


if __name__ == '__main__':
    run()
